@@ -5,60 +5,50 @@
 ## built circuits and all streams that succeed.
 ##
 
-import os
 import sys
-import stat
-import random
-
 from twisted.python import log
-from twisted.internet import reactor, defer
-from twisted.internet.endpoints import TCP4ClientEndpoint, UNIXClientEndpoint
-from zope.interface import implements
-
-try:
-    import psutil
-except ImportError:
-    psutil = None
-psutil = None
-
+from twisted.internet import reactor
 import txtorcon
 
+
 def logCircuit(circuit):
     path = '->'.join(map(lambda x: x.location.countrycode, circuit.path))
     log.msg('Circuit %d (%s) is %s for purpose "%s"' % (circuit.id, path, circuit.state, circuit.purpose))
 
+
 def logStream(stream, state):
     circ = ''
     if stream.circuit:
-        circ = ' via circuit %d' % stream.circuit.id
+        circ = ' via circuit %d (%s)' % (stream.circuit.id, '->'.join(map(lambda x: x.location.countrycode, stream.circuit.path)))
     proc = txtorcon.util.process_from_address(stream.source_addr, stream.source_port, state)
     if proc:
-        if psutil:
-            proc = ' from process "%s"' % (' '.join(proc.cmdline), )
-        else:
-            proc = ' from process "%s"' % (proc,)
-            
+        proc = ' from process "%s"' % (proc, )
+
     elif stream.source_addr == '(Tor_internal)':
         proc = ' for Tor internal use'
-        
+
     else:
         proc = ' from remote "%s:%s"' % (str(stream.source_addr), str(stream.source_port))
     log.msg('Stream %d to %s:%d attached%s%s' % (stream.id, stream.target_host, stream.target_port, circ, proc))
-    
+
+
 class StreamCircuitLogger(txtorcon.StreamListenerMixin, txtorcon.CircuitListenerMixin):
 
     def __init__(self, state):
         self.state = state
-    
+
     def stream_attach(self, stream, circuit):
         logStream(stream, self.state)
+
     def stream_failed(self, stream, reason, remote_reason):
         print 'Stream %d failed because "%s"' % (stream.id, remote_reason)
-    
+
     def circuit_built(self, circuit):
         logCircuit(circuit)
-    def circuit_failed(self, circuit, reason):
-        log.msg('circuit %d failed "%s"' % (circuit.id, reason))
+
+    def circuit_failed(self, circuit, kw):
+        log.msg('Circuit %d failed "%s"' % (circuit.id, kw['REASON']))
+
 
 def setup(state):
     log.msg('Connected to a Tor version %s' % state.protocol.version)
@@ -79,23 +69,15 @@ def setup(state):
     for c in state.circuits.values():
         logCircuit(c)
 
+
 def setup_failed(arg):
-    print "SETUP FAILED",arg
+    print "SETUP FAILED", arg
     log.err(arg)
     reactor.stop()
 
+
 log.startLogging(sys.stdout)
 
-d = None
-try:
-    if os.stat('/var/run/tor/control').st_mode & (stat.S_IRGRP | stat.S_IRUSR | stat.S_IROTH):
-        print "using control socket"
-        d = txtorcon.build_tor_connection(UNIXClientEndpoint(reactor, "/var/run/tor/control"))
-except OSError:
-    pass
-
-if d is None:
-    d = txtorcon.build_tor_connection(TCP4ClientEndpoint(reactor, "localhost", 9051))
-
+d = txtorcon.build_local_tor_connection(reactor)
 d.addCallback(setup).addErrback(setup_failed)
 reactor.run()
