@@ -1,23 +1,21 @@
-
 ## for now, this needs to be changed in setup.py also until I find a
 ## better solution
-__version__ = '0.7'
+__version__ = '0.8.0'
 __author__ = 'meejah'
 __contact__ = 'meejah@meejah.ca'
 __url__ = 'https://github.com/meejah/txtorcon'
 __license__ = 'MIT'
-__copyright__ = 'Copyright 2012'
+__copyright__ = 'Copyright 2012, 2013'
+
 
 from txtorcon.router import Router
 from txtorcon.circuit import Circuit
 from txtorcon.stream import Stream
 from txtorcon.torcontrolprotocol import TorControlProtocol, TorProtocolError, TorProtocolFactory, DEFAULT_VALUE
-from txtorcon.torstate import TorState, build_tor_connection
+from txtorcon.torstate import TorState, build_tor_connection, build_local_tor_connection
 from txtorcon.torconfig import TorConfig, HiddenService, TorProcessProtocol, TCPHiddenServiceEndpoint, launch_tor
 from txtorcon.torinfo import TorInfo
 from txtorcon.addrmap import AddrMap
-from txtorcon.addrmap import Addr
-from txtorcon.torinfo import TorInfo
 import util
 import interface
 from txtorcon.interface import *
@@ -25,17 +23,16 @@ from txtorcon.interface import *
 __all__ = ["Router",
            "Circuit",
            "Stream",
-           "TorControlProtocol", "TorProtocolError",
+           "TorControlProtocol", "TorProtocolError", "TorProtocolFactory",
            "TorState", "DEFAULT_VALUE",
            "TorInfo",
-           "build_tor_connection", "launch_tor",
+           "build_tor_connection", "build_local_tor_connection", "launch_tor",
            "TorConfig", "HiddenService", "TorProcessProtocol",
            "TorInfo",
            "TCPHiddenServiceEndpoint",
            "AddrMap",
            "util", "interface",
-
-           "ITorControlprotocol",
+           "ITorControlProtocol",
            "IStreamListener", "IStreamAttacher", "StreamListenerMixin",
            "ICircuitContainer", "ICircuitListener", "CircuitListenerMixin",
            "IRouterContainer", "IAddrListener"
