@@ -6,7 +6,7 @@ import functools
 from zope.interface import implements
 from twisted.trial import unittest
 from twisted.test import proto_helpers
-from twisted.internet import defer, error
+from twisted.internet import defer, error, task
 from twisted.python.failure import Failure
 from twisted.internet.interfaces import IReactorCore, IProtocolFactory, IReactorTCP
 
@@ -14,8 +14,6 @@ from txtorcon import TorControlProtocol, ITorControlProtocol, TorConfig, DEFAULT
 
 from txtorcon.util import delete_file_or_tree
 
-def do_nothing(*args):
-    pass
 
 class FakeControlProtocol:
     """
@@ -31,7 +29,7 @@ class FakeControlProtocol:
     Deferred is already-fired when get_conf runs, there's a Very Good
     Chance (always?) that the callback just runs right away.
     """
-    
+
     implements(ITorControlProtocol)     # actually, just get_info_raw
 
     def __init__(self, answers):
@@ -51,7 +49,7 @@ class FakeControlProtocol:
             d = defer.Deferred()
             self.pending.append(d)
             return d
-        
+
         d = defer.succeed(self.answers[0])
         self.answers = self.answers[1:]
         return d
@@ -61,22 +59,24 @@ class FakeControlProtocol:
             d = defer.Deferred()
             self.pending.append(d)
             return d
-        
+
         d = defer.succeed(self.answers[0])
         self.answers = self.answers[1:]
         return d
 
     get_conf_raw = get_conf             # up to test author ensure the answer is a raw string
-    
+
     def set_conf(self, *args):
         for i in range(0, len(args), 2):
-            self.sets.append((args[i], args[i+1]))
+            self.sets.append((args[i], args[i + 1]))
         return defer.succeed('OK')
 
     def add_event_listener(self, nm, cb):
         self.events[nm] = cb
 
+
 class CheckAnswer:
+
     def __init__(self, test, ans):
         self.answer = ans
         self.test = test
@@ -84,49 +84,41 @@ class CheckAnswer:
     def __call__(self, x):
         self.test.assertEqual(x, self.answer)
 
+
 class ConfigTests(unittest.TestCase):
     """
-    FIXME hmm, this all seems a little convoluted to test errors? 
+    FIXME hmm, this all seems a little convoluted to test errors?
     Maybe not that bad.
     """
-    
+
     def setUp(self):
         self.protocol = FakeControlProtocol([])
 
     def test_boolean_parse_error(self):
-        self.protocol.answers.append('''config/names=
-foo Boolean
-OK''')
-        self.protocol.answers.append({'foo':'bar'})
-        conf = TorConfig(self.protocol)
+        self.protocol.answers.append('config/names=\nfoo Boolean\nOK')
+        self.protocol.answers.append({'foo': 'bar'})
+        TorConfig(self.protocol)
         errs = self.flushLoggedErrors(ValueError)
         self.assertEqual(len(errs), 1)
         ## dunno if asserting strings in messages is a good idea...
         self.assertTrue('invalid literal' in errs[0].getErrorMessage())
-    
+
     def test_boolean_parser(self):
-        self.protocol.answers.append('''config/names=
-foo Boolean
-bar Boolean
-OK''')
-        self.protocol.answers.append({'foo':'0'})
-        self.protocol.answers.append({'bar':'1'})
+        self.protocol.answers.append('config/names=\nfoo Boolean\nbar Boolean\nOK')
+        self.protocol.answers.append({'foo': '0'})
+        self.protocol.answers.append({'bar': '1'})
         ## FIXME does a Tor controller only ever send "0" and "1" for
         ## true/false? Or do we need to accept others?
-        
+
         conf = TorConfig(self.protocol)
         self.assertTrue(conf.foo is False)
         self.assertTrue(conf.bar is True)
 
     def test_boolean_auto_parser(self):
-        self.protocol.answers.append('''config/names=
-foo Boolean+Auto
-bar Boolean+Auto
-baz Boolean+Auto
-OK''')
-        self.protocol.answers.append({'foo':'0'})
-        self.protocol.answers.append({'bar':'1'})
-        self.protocol.answers.append({'baz':'auto'})
+        self.protocol.answers.append('config/names=\nfoo Boolean+Auto\nbar Boolean+Auto\nbaz Boolean+Auto\nOK')
+        self.protocol.answers.append({'foo': '0'})
+        self.protocol.answers.append({'bar': '1'})
+        self.protocol.answers.append({'baz': 'auto'})
 
         conf = TorConfig(self.protocol)
         self.assertTrue(conf.foo is 0)
@@ -134,173 +126,143 @@ OK''')
         self.assertTrue(conf.baz is -1)
 
     def test_string_parser(self):
-        self.protocol.answers.append('''config/names=
-foo String
-OK''')
-        self.protocol.answers.append({'foo':'bar'})
+        self.protocol.answers.append('config/names=\nfoo String\nOK')
+        self.protocol.answers.append({'foo': 'bar'})
         conf = TorConfig(self.protocol)
-        self.assertEqual(conf.foo, 'bar')        
+        self.assertEqual(conf.foo, 'bar')
 
     def test_int_parser(self):
-        self.protocol.answers.append('''config/names=
-foo Integer
-OK''')
-        self.protocol.answers.append({'foo':'123'})
+        self.protocol.answers.append('config/names=\nfoo Integer\nOK')
+        self.protocol.answers.append({'foo': '123'})
         conf = TorConfig(self.protocol)
         self.assertEqual(conf.foo, 123)
-        
+
     def test_int_parser_error(self):
-        self.protocol.answers.append('''config/names=
-foo Integer
-OK''')
-        self.protocol.answers.append({'foo':'123foo'})
-        conf = TorConfig(self.protocol)
+        self.protocol.answers.append('config/names=\nfoo Integer\nOK')
+        self.protocol.answers.append({'foo': '123foo'})
+        TorConfig(self.protocol)
         errs = self.flushLoggedErrors(ValueError)
         self.assertEqual(len(errs), 1)
         self.assertTrue(isinstance(errs[0].value, ValueError))
 
     def test_int_parser_error_2(self):
-        self.protocol.answers.append('''config/names=
-foo Integer
-OK''')
-        self.protocol.answers.append({'foo':'1.23'})
-        conf = TorConfig(self.protocol)
+        self.protocol.answers.append('config/names=\nfoo Integer\nOK')
+        self.protocol.answers.append({'foo': '1.23'})
+        TorConfig(self.protocol)
         errs = self.flushLoggedErrors(ValueError)
         self.assertEqual(len(errs), 1)
         self.assertTrue(isinstance(errs[0].value, ValueError))
 
     def test_linelist_parser(self):
-        self.protocol.answers.append('''config/names=
-foo LineList
-OK''')
-        self.protocol.answers.append({'foo':'bar\nbaz'})
+        self.protocol.answers.append('config/names=\nfoo LineList\nOK')
+        self.protocol.answers.append({'foo': 'bar\nbaz'})
         conf = TorConfig(self.protocol)
         self.assertEqual(conf.foo, ['bar', 'baz'])
 
     def test_listlist_parser_with_list(self):
         self.protocol.answers.append('config/names=\nfoo LineList\nOK')
-        self.protocol.answers.append({'foo': [1,2,3]})
-        
+        self.protocol.answers.append({'foo': [1, 2, 3]})
+
         conf = TorConfig(self.protocol)
         self.assertEqual(conf.foo, ['1', '2', '3'])
 
     def test_float_parser(self):
-        self.protocol.answers.append('''config/names=
-foo Float
-OK''')
-        self.protocol.answers.append({'foo':'1.23'})
+        self.protocol.answers.append('config/names=\nfoo Float\nOK')
+        self.protocol.answers.append({'foo': '1.23'})
         conf = TorConfig(self.protocol)
         self.assertEqual(conf.foo, 1.23)
 
     def test_float_parser_error(self):
-        self.protocol.answers.append('''config/names=
-foo Float
-OK''')
-        self.protocol.answers.append({'foo':'1.23fff'})
-        conf = TorConfig(self.protocol)
+        self.protocol.answers.append('config/names=\nfoo Float\nOK')
+        self.protocol.answers.append({'foo': '1.23fff'})
+        TorConfig(self.protocol)
         errs = self.flushLoggedErrors(ValueError)
         self.assertEqual(len(errs), 1)
         self.assertTrue(isinstance(errs[0].value, ValueError))
 
     def test_list(self):
-        self.protocol.answers.append('''config/names=
-bing CommaList
-OK''')
-        self.protocol.answers.append({'bing':'foo,bar,baz'})
+        self.protocol.answers.append('config/names=\nbing CommaList\nOK')
+        self.protocol.answers.append({'bing': 'foo,bar,baz'})
         conf = TorConfig(self.protocol)
-        self.assertEqual(conf.config['bing'], ['foo','bar','baz'])
-#        self.assertEqual(conf.bing, ['foo','bar','baz'])
-        
+        self.assertEqual(conf.config['bing'], ['foo', 'bar', 'baz'])
+        # self.assertEqual(conf.bing, ['foo','bar','baz'])
+
     def test_single_list(self):
-        self.protocol.answers.append('''config/names=
-bing CommaList
-OK''')
-        self.protocol.answers.append({'bing':'foo'})
+        self.protocol.answers.append('config/names=\nbing CommaList\nOK')
+        self.protocol.answers.append({'bing': 'foo'})
         conf = TorConfig(self.protocol)
         self.assertEqual(conf.config['bing'], ['foo'])
 
     def test_multi_list_space(self):
-        self.protocol.answers.append('''config/names=
-bing CommaList
-OK''')
-        self.protocol.answers.append({'bing':'foo, bar , baz'})
+        self.protocol.answers.append('config/names=\nbing CommaList\nOK')
+        self.protocol.answers.append({'bing': 'foo, bar , baz'})
         conf = TorConfig(self.protocol)
         self.assertEqual(conf.bing, ['foo', 'bar', 'baz'])
 
     def test_descriptor_access(self):
-        self.protocol.answers.append('''config/names=
-bing CommaList
-OK''')
-        self.protocol.answers.append({'bing':'foo,bar'})
-        
+        self.protocol.answers.append('config/names=\nbing CommaList\nOK')
+        self.protocol.answers.append({'bing': 'foo,bar'})
+
         conf = TorConfig(self.protocol)
-        self.assertEqual(conf.config['bing'], ['foo','bar'])
-        self.assertEqual(conf.bing, ['foo','bar'])
+        self.assertEqual(conf.config['bing'], ['foo', 'bar'])
+        self.assertEqual(conf.bing, ['foo', 'bar'])
 
         self.protocol.answers.append('250 OK')
-        conf.bing = ['a','b']
-        self.assertEqual(conf.bing, ['foo','bar'])
-        
+        conf.bing = ['a', 'b']
+        self.assertEqual(conf.bing, ['foo', 'bar'])
+
         d = conf.save()
+
         def confirm(conf):
-            self.assertEqual(conf.config['bing'], ['a','b'])
-            self.assertEqual(conf.bing, ['a','b'])
+            self.assertEqual(conf.config['bing'], ['a', 'b'])
+            self.assertEqual(conf.bing, ['a', 'b'])
+
         d.addCallbacks(confirm, self.fail)
         return d
 
     def test_unknown_descriptor(self):
-        self.protocol.answers.append('''config/names=
-bing CommaList
-OK''')
-        self.protocol.answers.append({'bing':'foo'})
-        
+        self.protocol.answers.append('config/names=\nbing CommaList\nOK')
+        self.protocol.answers.append({'bing': 'foo'})
+
         conf = TorConfig(self.protocol)
         try:
             conf.foo
             self.assertTrue(False)
         except KeyError, e:
             self.assertTrue('foo' in str(e))
-            
+
     def test_invalid_parser(self):
-        self.protocol.answers.append('''config/names=
-SomethingExciting NonExistantParserType
-OK''')
-        conf = TorConfig(self.protocol)
+        self.protocol.answers.append('config/names=\nSomethingExciting NonExistantParserType\nOK')
+        TorConfig(self.protocol)
         errs = self.flushLoggedErrors()
         self.assertEqual(len(errs), 1)
         self.assertTrue('NonExistantParserType' in str(errs[0]))
 
     def foo(self, *args):
-        print "FOOO",args
+        print "FOOO", args
 
     def test_slutty_postbootstrap(self):
         # test that doPostbootstrap still works in "slutty" mode
-        self.protocol.answers.append('''config/names=
-ORPort Port
-OK''')
+        self.protocol.answers.append('config/names=\nORPort Port\nOK')
         ## we can't answer right away, or we do all the _do_setup
         ## callbacks before _setup_ is set -- but we need to do an
         ## answer callback after that to trigger this bug
 
         conf = TorConfig(self.protocol)
-        self.assertTrue(conf.__dict__.has_key('_setup_'))
-        self.protocol.answer_pending({'ORPort':1})
+        self.assertTrue('_setup_' in conf.__dict__)
+        self.protocol.answer_pending({'ORPort': 1})
 
     def test_immediate_bootstrap(self):
         self.protocol.post_bootstrap = None
-        self.protocol.answers.append('''config/names=
-foo Boolean
-OK''')
-        self.protocol.answers.append({'foo':'0'})
+        self.protocol.answers.append('config/names=\nfoo Boolean\nOK')
+        self.protocol.answers.append({'foo': '0'})
         conf = TorConfig(self.protocol)
-        self.assertTrue(conf.config.has_key('foo'))
+        self.assertTrue('foo' in conf.config)
 
     def test_multiple_orports(self):
         self.protocol.post_bootstrap = None
-        self.protocol.answers.append('''config/names=
-OrPort CommaList
-OK''')
-        self.protocol.answers.append({'OrPort':'1234'})
+        self.protocol.answers.append('config/names=\nOrPort CommaList\nOK')
+        self.protocol.answers.append({'OrPort': '1234'})
         conf = TorConfig(self.protocol)
         conf.OrPort = ['1234', '4321']
         conf.save()
@@ -308,11 +270,9 @@ OK''')
                                               ('OrPort', '4321')])
 
     def test_set_multiple(self):
-        self.protocol.answers.append('''config/names=
-AwesomeKey String
-OK''')
-        self.protocol.answers.append({'AwesomeKey':'foo'})
-        
+        self.protocol.answers.append('config/names=\nAwesomeKey String\nOK')
+        self.protocol.answers.append({'AwesomeKey': 'foo'})
+
         conf = TorConfig(self.protocol)
         conf.awesomekey
         conf.awesomekey = 'baz'
@@ -321,35 +281,30 @@ OK''')
         conf.awesomekey = 'pac man'
 
         conf.save()
-        
+
         self.assertEqual(len(self.protocol.sets), 1)
         self.assertEqual(self.protocol.sets[0], ('AwesomeKey', 'pac man'))
 
     def test_log_double_save(self):
-        self.protocol.answers.append('''config/names=
-Log LineList
-Foo String
-OK''')
-        self.protocol.answers.append({'Log':'notice file /var/log/tor/notices.log'})
-        self.protocol.answers.append({'Foo':'foo'})
+        self.protocol.answers.append('config/names=\nLog LineList\nFoo String\nOK''')
+        self.protocol.answers.append({'Log': 'notice file /var/log/tor/notices.log'})
+        self.protocol.answers.append({'Foo': 'foo'})
         conf = TorConfig(self.protocol)
 
         conf.log.append('info file /tmp/foo.log')
         conf.foo = 'bar'
         self.assertTrue(conf.needs_save())
         conf.save()
-        conf.save()                     # just for the code coverage...
-        
+        conf.save()  # just for the code coverage...
+
         self.assertTrue(not conf.needs_save())
         self.protocol.sets = []
         conf.save()
         self.assertEqual(self.protocol.sets, [])
 
     def test_set_save_modify(self):
-        self.protocol.answers.append('''config/names=
-Log LineList
-OK''')
-        self.protocol.answers.append({'Log':'notice file /var/log/tor/notices.log'})
+        self.protocol.answers.append('config/names=\nLog LineList\nOK')
+        self.protocol.answers.append({'Log': 'notice file /var/log/tor/notices.log'})
         conf = TorConfig(self.protocol)
 
         conf.log = []
@@ -360,11 +315,9 @@ OK''')
         self.assertTrue(conf.needs_save())
 
     def test_proper_sets(self):
-        self.protocol.answers.append('''config/names=
-Log LineList
-OK''')
-        self.protocol.answers.append({'Log':'foo'})
-        
+        self.protocol.answers.append('config/names=\nLog LineList\nOK')
+        self.protocol.answers.append({'Log': 'foo'})
+
         conf = TorConfig(self.protocol)
         conf.log.append('bar')
         conf.save()
@@ -372,15 +325,14 @@ OK''')
         self.assertEqual(len(self.protocol.sets), 2)
         self.assertEqual(self.protocol.sets[0], ('Log', 'foo'))
         self.assertEqual(self.protocol.sets[1], ('Log', 'bar'))
-        
+
+
 class LogTests(unittest.TestCase):
-    
+
     def setUp(self):
         self.protocol = FakeControlProtocol([])
-        self.protocol.answers.append('''config/names=
-Log LineList
-OK''')
-        self.protocol.answers.append({'Log':'notice file /var/log/tor/notices.log'})
+        self.protocol.answers.append('config/names=\nLog LineList\nOK''')
+        self.protocol.answers.append({'Log': 'notice file /var/log/tor/notices.log'})
 
     def test_log_set(self):
         conf = TorConfig(self.protocol)
@@ -419,7 +371,7 @@ OK''')
         conf.save()
 
         self.assertEqual(self.protocol.sets[0], ('Log', 'info file /tmp/foo.log'))
-        
+
     def test_log_set_pop(self):
         conf = TorConfig(self.protocol)
 
@@ -430,7 +382,7 @@ OK''')
 
         self.assertEqual(len(conf.log), 0)
         self.assertEqual(len(self.protocol.sets), 0)
-        
+
     def test_log_set_extend(self):
         conf = TorConfig(self.protocol)
 
@@ -443,7 +395,7 @@ OK''')
         self.assertEqual(len(self.protocol.sets), 2)
         self.assertEqual(self.protocol.sets[0], ('Log', 'notice file /var/log/tor/notices.log'))
         self.assertEqual(self.protocol.sets[1], ('Log', 'info file /tmp/foo'))
-        
+
     def test_log_set_insert(self):
         conf = TorConfig(self.protocol)
 
@@ -456,7 +408,7 @@ OK''')
         self.assertEqual(len(self.protocol.sets), 2)
         self.assertEqual(self.protocol.sets[1], ('Log', 'notice file /var/log/tor/notices.log'))
         self.assertEqual(self.protocol.sets[0], ('Log', 'info file /tmp/foo'))
-        
+
     def test_log_set_remove(self):
         conf = TorConfig(self.protocol)
 
@@ -496,18 +448,16 @@ class EventTests(unittest.TestCase):
     def test_conf_changed(self):
         control = FakeControlProtocol([])
         config = TorConfig(control)
-        self.assertTrue(control.events.has_key('CONF_CHANGED'))
+        self.assertTrue('CONF_CHANGED' in control.events)
 
         control.events['CONF_CHANGED']('Foo=bar\nBar')
         self.assertEqual(len(config.config), 2)
         self.assertEqual(config.Foo, 'bar')
         self.assertEqual(config.Bar, DEFAULT_VALUE)
 
-        
-
 
 class CreateTorrcTests(unittest.TestCase):
-    
+
     def test_create_torrc(self):
         config = TorConfig()
         config.SocksPort = 1234
@@ -524,8 +474,10 @@ Log 80 127.0.0.1:80
 Log 90 127.0.0.1:90
 SocksPort 1234
 ''')
-        
+
+
 class HiddenServiceTests(unittest.TestCase):
+
     def setUp(self):
         self.protocol = FakeControlProtocol([])
         self.protocol.answers.append('''config/names=
@@ -536,9 +488,9 @@ OK''')
 
     def test_options_hidden(self):
         self.protocol.answers.append('HiddenServiceDir=/fake/path\nHiddenServicePort=80 127.0.0.1:1234\n')
-        
+
         conf = TorConfig(self.protocol)
-        self.assertTrue(not conf.config.has_key('HiddenServiceOptions'))
+        self.assertTrue('HiddenServiceOptions' not in conf.config)
         self.assertEqual(len(conf.HiddenServices), 1)
 
         self.assertTrue(not conf.needs_save())
@@ -565,7 +517,7 @@ OK''')
     def test_onion_keys(self):
         self.protocol.answers.append('HiddenServiceDir=/fake/path\n')
         d = tempfile.mkdtemp()
-        
+
         try:
             open(os.path.join(d, 'hostname'), 'w').write('public')
             open(os.path.join(d, 'private_key'), 'w').write('private')
@@ -575,17 +527,17 @@ OK''')
 
             self.assertEqual(hs.hostname, 'public')
             self.assertEqual(hs.private_key, 'private')
-            
+
         finally:
             shutil.rmtree(d, ignore_errors=True)
 
     def test_modify_hidden_service(self):
         self.protocol.answers.append('HiddenServiceDir=/fake/path\nHiddenServicePort=80 127.0.0.1:1234\n')
-        
+
         conf = TorConfig(self.protocol)
         conf.hiddenservices[0].version = 3
         self.assertTrue(conf.needs_save())
-        
+
     def test_add_hidden_service_to_empty_config(self):
         conf = TorConfig()
         h = HiddenService(conf, '/fake/path', ['80 127.0.0.1:1234'], '', 3)
@@ -611,12 +563,12 @@ HiddenServicePort=90 127.0.0.1:2345''')
         self.assertEqual(conf.hiddenservices[0].authorize_client, 'basic')
         self.assertEqual(len(conf.hiddenservices[0].ports), 1)
         self.assertEqual(conf.hiddenservices[0].ports[0], '80 127.0.0.1:1234')
-        
+
         self.assertEqual(conf.hiddenservices[1].dir, '/some/other/fake/path')
         self.assertEqual(len(conf.hiddenservices[1].ports), 2)
         self.assertEqual(conf.hiddenservices[1].ports[0], '80 127.0.0.1:1234')
         self.assertEqual(conf.hiddenservices[1].ports[1], '90 127.0.0.1:2345')
-        
+
     def test_hidden_service_parse_error(self):
         conf = TorConfig(FakeControlProtocol(['config/names=']))
         try:
@@ -627,7 +579,7 @@ HiddenServicePort=90 127.0.0.1:2345''')
 
     def test_multiple_modify_hidden_service(self):
         self.protocol.answers.append('HiddenServiceDir=/fake/path\nHiddenServicePort=80 127.0.0.1:1234\n')
-        
+
         conf = TorConfig(self.protocol)
         conf.hiddenservices[0].version = 3
         self.assertTrue(conf.needs_save())
@@ -640,10 +592,10 @@ HiddenServicePort=90 127.0.0.1:2345''')
         self.assertEqual(self.protocol.sets[0], ('HiddenServiceDir', '/fake/path'))
         self.assertEqual(self.protocol.sets[1], ('HiddenServicePort', '80 127.0.0.1:1234'))
         self.assertEqual(self.protocol.sets[2], ('HiddenServiceVersion', '5'))
-        
+
     def test_set_save_modify(self):
         self.protocol.answers.append('')
-        
+
         conf = TorConfig(self.protocol)
 
         conf.hiddenservices = [HiddenService(conf, '/fake/path', ['80 127.0.0.1:1234'], '', 3)]
@@ -661,10 +613,12 @@ HiddenServicePort=90 127.0.0.1:2345''')
         conf.hiddenservices[0].ports.append('90 127.0.0.1:2345')
         self.assertTrue(conf.needs_save())
 
-class FakeReactor:
+
+class FakeReactor(task.Clock):
     implements(IReactorCore)
 
     def __init__(self, test, trans, on_protocol):
+        super(FakeReactor, self).__init__()
         self.test = test
         self.transport = trans
         self.on_protocol = on_protocol
@@ -674,7 +628,7 @@ class FakeReactor:
         self.protocol.makeConnection(self.transport)
         self.on_protocol(self.protocol)
         return self.transport
-        
+
     def addSystemEventTrigger(self, *args):
         self.test.assertEqual(args[0], 'before')
         self.test.assertEqual(args[1], 'shutdown')
@@ -682,10 +636,11 @@ class FakeReactor:
         ## nuke it right away to avoid polluting /tmp but calling the
         ## callback now.
         args[2]()
-        
+
     def removeSystemEventTrigger(self, id):
         pass
 
+
 class FakeProcessTransport(proto_helpers.StringTransportWithDisconnection):
 
     pid = -1
@@ -696,13 +651,31 @@ class FakeProcessTransport(proto_helpers.StringTransportWithDisconnection):
         self.protocol.dataReceived('250 OK\r\n')
         self.protocol.dataReceived('650 STATUS_CLIENT NOTICE BOOTSTRAP PROGRESS=90 TAG=circuit_create SUMMARY="Establishing a Tor circuit"\r\n')
         self.protocol.dataReceived('650 STATUS_CLIENT NOTICE BOOTSTRAP PROGRESS=100 TAG=done SUMMARY="Done"\r\n')
-  
+
+
+class FakeProcessTransportNeverBootstraps(proto_helpers.StringTransportWithDisconnection):
+
+    pid = -1
+
+    def closeStdin(self):
+        self.protocol.dataReceived('250 OK\r\n')
+        self.protocol.dataReceived('250 OK\r\n')
+        self.protocol.dataReceived('250 OK\r\n')
+        self.protocol.dataReceived('650 STATUS_CLIENT NOTICE BOOTSTRAP PROGRESS=90 TAG=circuit_create SUMMARY="Establishing a Tor circuit"\r\n')
+
+
 class LaunchTorTests(unittest.TestCase):
+
     def setUp(self):
         self.protocol = TorControlProtocol()
-        self.protocol.connectionMade = do_nothing
+        self.protocol.connectionMade = lambda: None
         self.transport = proto_helpers.StringTransport()
         self.protocol.makeConnection(self.transport)
+        self.clock = task.Clock()
+
+    def setup_complete_with_timer(self, proto):
+        proto._check_timeout.stop()
+        proto.checkTimeout()
 
     def setup_complete_no_errors(self, proto, config):
         todel = proto.to_delete
@@ -711,6 +684,7 @@ class LaunchTorTests(unittest.TestCase):
         self.assertEqual(len(proto.to_delete), 0)
         for f in todel:
             self.assertTrue(not os.path.exists(f))
+        self.assertEqual(proto._timeout_delayed_call, None)
 
         ## make sure we set up the config to track the created tor
         ## protocol connection
@@ -745,8 +719,8 @@ class LaunchTorTests(unittest.TestCase):
                 self.test.assertEqual(self.expected[0], (percent, tag, summary))
                 self.expected = self.expected[1:]
                 self.test.assertTrue('"' not in summary)
-                self.test.assertTrue(percent >= 0 and percent <= 100)            
-            
+                self.test.assertTrue(percent >= 0 and percent <= 100)
+
         def on_protocol(proto):
             proto.outReceived('Bootstrapped 100%\n')
             proto.progress = OnProgress(self, [(90, 'circuit_create', 'Establishing a Tor circuit'),
@@ -759,12 +733,12 @@ class LaunchTorTests(unittest.TestCase):
         d = launch_tor(config, FakeReactor(self, trans, on_protocol), connection_creator=creator)
         d.addCallback(self.setup_complete_no_errors, config)
         return d
-        
+
     def check_setup_failure(self, fail):
         self.assertTrue("with error-code 12" in fail.getErrorMessage())
         ## cancel the errback chain, we wanted this
         return None
-                
+
     def test_launch_tor_fails(self):
         config = TorConfig()
         config.OrPort = 1234
@@ -775,24 +749,61 @@ class LaunchTorTests(unittest.TestCase):
             proto.makeConnection(trans)
             proto.post_bootstrap.callback(proto)
             return proto.post_bootstrap
-            
+
         def on_protocol(proto):
             proto.outReceived('Bootstrapped 100%\n')
-            
+
         trans = FakeProcessTransport()
         trans.protocol = self.protocol
         self.othertrans = trans
         creator = functools.partial(connector, self.protocol, self.transport)
         d = launch_tor(config, FakeReactor(self, trans, on_protocol), connection_creator=creator)
         d.addCallback(self.setup_complete_fails)
-        d.addErrback(self.check_setup_failure)
-        return d
+        return self.assertFailure(d, Exception)
+
+    def test_launch_with_timeout(self):
+        config = TorConfig()
+        config.OrPort = 1234
+        config.SocksPort = 9999
+        timeout = 5
+
+        def connector(proto, trans):
+            proto._set_valid_events('STATUS_CLIENT')
+            proto.makeConnection(trans)
+            proto.post_bootstrap.callback(proto)
+            return proto.post_bootstrap
+
+        class OnProgress:
+            def __init__(self, test, expected):
+                self.test = test
+                self.expected = expected
+
+            def __call__(self, percent, tag, summary):
+                self.test.assertEqual(self.expected[0], (percent, tag, summary))
+                self.expected = self.expected[1:]
+                self.test.assertTrue('"' not in summary)
+                self.test.assertTrue(percent >= 0 and percent <= 100)
+
+        def on_protocol(proto):
+            proto.outReceived('Bootstrapped 100%\n')
+
+        trans = FakeProcessTransportNeverBootstraps()
+        trans.protocol = self.protocol
+        self.othertrans = trans
+        creator = functools.partial(connector, self.protocol, self.transport)
+        react = FakeReactor(self, trans, on_protocol)
+        d = launch_tor(config, react, connection_creator=creator,
+                       timeout=timeout)
+        rtn = self.assertFailure(d, RuntimeError, "Timed out waiting for Tor to launch.")
+        # FakeReactor is a task.Clock subclass and +1 just to be sure
+        react.advance(timeout + 1)
+        return rtn
 
     def setup_fails_stderr(self, fail):
         self.assertTrue('Something went horribly wrong!' in fail.getErrorMessage())
         ## cancel the errback chain, we wanted this
         return None
-        
+
     def test_tor_produces_stderr_output(self):
         config = TorConfig()
         config.OrPort = 1234
@@ -803,10 +814,10 @@ class LaunchTorTests(unittest.TestCase):
             proto.makeConnection(trans)
             proto.post_bootstrap.callback(proto)
             return proto.post_bootstrap
-            
+
         def on_protocol(proto):
             proto.errReceived('Something went horribly wrong!\n')
-            
+
         trans = FakeProcessTransport()
         trans.protocol = self.protocol
         self.othertrans = trans
@@ -815,14 +826,14 @@ class LaunchTorTests(unittest.TestCase):
         d.addCallback(self.fail)        # should't get callback
         d.addErrback(self.setup_fails_stderr)
         return d
-        
+
     def test_tor_connection_fails(self):
         """
         We fail to connect once, and then successfully connect --
         testing whether we're retrying properly on each Bootstrapped
         line from stdout.
         """
-        
+
         config = TorConfig()
         config.OrPort = 1234
         config.SocksPort = 9999
@@ -843,15 +854,14 @@ class LaunchTorTests(unittest.TestCase):
         def on_protocol(proto):
             proto.outReceived('Bootstrapped 90%\n')
             proto.outReceived('Bootstrapped 100%\n')
-            
+
         trans = FakeProcessTransport()
         trans.protocol = self.protocol
         self.othertrans = trans
         creator = functools.partial(Connector(), self.protocol, self.transport)
         d = launch_tor(config, FakeReactor(self, trans, on_protocol), connection_creator=creator)
         d.addCallback(self.setup_complete_fails)
-        d.addErrback(self.check_setup_failure)
-        return d
+        return self.assertFailure(d, Exception)
 
     def test_tor_connection_user_data_dir(self):
         """
@@ -879,10 +889,12 @@ class LaunchTorTests(unittest.TestCase):
         self.othertrans = trans
         creator = functools.partial(Connector(), self.protocol, self.transport)
         d = launch_tor(config, FakeReactor(self, trans, on_protocol), connection_creator=creator)
+
         def still_have_data_dir(proto, tester):
-            proto.cleanup()             # FIXME? not really unit-testy as this is sort of internal function
+            proto.cleanup()  # FIXME? not really unit-testy as this is sort of internal function
             tester.assertTrue(os.path.exists(my_dir))
             delete_file_or_tree(my_dir)
+
         d.addCallback(still_have_data_dir, self)
         d.addErrback(self.fail)
         return d
@@ -957,11 +969,11 @@ class LaunchTorTests(unittest.TestCase):
     def confirm_progress(self, exp, *args, **kwargs):
         self.assertEqual(exp, args)
         self.got_progress = True
-        
+
     def test_progress_updates(self):
         from txtorcon.torconfig import TorProcessProtocol
 
-        self.got_progress = False;
+        self.got_progress = False
         proto = TorProcessProtocol(None, functools.partial(self.confirm_progress,
                                                            (10, 'tag', 'summary')))
         proto.progress(10, 'tag', 'summary')
@@ -979,21 +991,25 @@ class FakeProtocolFactory:
 
     def buildProtocol(self, addr):
         return None
-    
+
     def doStart(self):
         return None
 
     def doStop(self):
         return None
 
+
 class FakeListeningPort(object):
+
     def startListening(self):
         print "startListening"
+
     def stopListening(self):
         print "stopListening"
+
     def getHost(self):
         return "host"
-        
+
 
 class FakeReactorTcp(object):
     implements(IReactorTCP)
@@ -1004,13 +1020,14 @@ class FakeReactorTcp(object):
         if self.failures > 0:
             self.failures -= 1
             raise error.CannotListenError(None, None, None)
-        
+
         return FakeListeningPort()
-    
+
     def connectTCP(self, host, port, factory, **kwargs):
-        print "listenTCP",port,factory,kwards
+        print "listenTCP", port, factory, kwargs
         return FakeListeningPort()
-        
+
+
 class EndpointTests(unittest.TestCase):
 
     def setUp(self):
@@ -1021,25 +1038,30 @@ class EndpointTests(unittest.TestCase):
     def test_basic(self):
         ep = TCPHiddenServiceEndpoint(self.reactor, self.config, 123)
         d = ep.listen(FakeProtocolFactory())
+        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual\nOK')
+        self.protocol.answers.append('HiddenServiceOptions')
+        self.config.bootstrap()
+        return d
 
+    def test_already_bootstrapped(self):
         self.protocol.answers.append('''config/names=
 HiddenServiceOptions Virtual
 OK''')
         self.protocol.answers.append('HiddenServiceOptions')
-        
+
         self.config.bootstrap()
 
+        ep = TCPHiddenServiceEndpoint(self.reactor, self.config, 123)
+        d = ep.listen(FakeProtocolFactory())
         return d
 
     def test_explicit_data_dir(self):
         ep = TCPHiddenServiceEndpoint(self.reactor, self.config, 123, '/mumble/mumble')
         d = ep.listen(FakeProtocolFactory())
 
-        self.protocol.answers.append('''config/names=
-HiddenServiceOptions Virtual
-OK''')
+        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual\nOK')
         self.protocol.answers.append('HiddenServiceOptions')
-        
+
         self.config.bootstrap()
 
         return d
@@ -1064,13 +1086,9 @@ OK''')
         ep = TCPHiddenServiceEndpoint(self.reactor, self.config, 123)
         d = ep.listen(FakeProtocolFactory())
 
-        self.protocol.answers.append('''config/names=
-HiddenServiceOptions Virtual
-OK''')
+        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual\nOK')
         self.protocol.answers.append('HiddenServiceOptions')
-        
         self.config.bootstrap()
-
         return d
 
     def check_error(self, failure):
@@ -1082,13 +1100,8 @@ OK''')
         ep = TCPHiddenServiceEndpoint(self.reactor, self.config, 123)
         d = ep.listen(FakeProtocolFactory())
 
-        self.protocol.answers.append('''config/names=
-HiddenServiceOptions Virtual
-OK''')
+        self.protocol.answers.append('config/names=\nHiddenServiceOptions Virtual\nOK')
         self.protocol.answers.append('HiddenServiceOptions')
-        
         self.config.bootstrap()
-
         d.addErrback(self.check_error)
-
         return d
