@@ -1,35 +1,34 @@
 from __future__ import with_statement
 
-from twisted.python import log, failure
-from twisted.internet import defer, error, protocol
-from twisted.internet.interfaces import IProtocolFactory, IStreamServerEndpoint
-from twisted.internet.endpoints import TCP4ClientEndpoint, TCP4ServerEndpoint
-from twisted.protocols.basic import LineOnlyReceiver
-from zope.interface import implements
-
-## outside this module, you can do "from txtorcon import Stream" etc.
-from txtorcon.stream import Stream
-from txtorcon.circuit import Circuit
-from txtorcon.router import Router
-from txtorcon.addrmap import AddrMap
-from txtorcon.torcontrolprotocol import parse_keywords, DEFAULT_VALUE, TorProtocolFactory
-from txtorcon.util import delete_file_or_tree, find_keywords
-from txtorcon.log import txtorlog
-
-from txtorcon.interface import ITorControlProtocol, IRouterContainer, ICircuitListener
-from txtorcon.interface import ICircuitContainer, IStreamListener, IStreamAttacher
-from spaghetti import FSM, State, Transition
-
 import os
 import sys
 import string
-import itertools
 import types
 import functools
 import random
 import tempfile
 from StringIO import StringIO
 import shlex
+if sys.platform in ('linux2', 'darwin'):
+    import pwd
+
+from twisted.python import log
+from twisted.internet import defer, error, protocol
+from twisted.internet.interfaces import IStreamServerEndpoint, IReactorTime
+from twisted.internet.endpoints import TCP4ClientEndpoint, TCP4ServerEndpoint
+from zope.interface import implements
+
+from txtorcon.torcontrolprotocol import parse_keywords, TorProtocolFactory
+from txtorcon.util import delete_file_or_tree, find_keywords, find_tor_binary
+from txtorcon.log import txtorlog
+from txtorcon.interface import ITorControlProtocol
+
+
+class TorNotFound(RuntimeError):
+    """
+    Raised by launch_tor() in case the tor binary was unspecified and could
+    not be found by consulting the shell.
+    """
 
 
 class TCPHiddenServiceEndpoint(object):
@@ -162,8 +161,8 @@ class TCPHiddenServiceEndpoint(object):
         :api:`twisted.internet.interfaces.IPort` instance that also
         has at least `onion_uri` and `onion_private_key` members set
         (both strings). Really this is just what
-        :api:`twisted.internet.endpoint.TCP4ServerEndpoint
-        <TCP4ServerEndpoint>` returned, with a few members set. At
+        :api:`twisted.internet.endpoints.TCP4ServerEndpoint
+        <TCP4ServerEndpoint>`.listen() returned, with a few members set. At
         this point, Tor will have fully started up and successfully
         accepted the hidden service's config.
         """
@@ -204,7 +203,7 @@ class TCPHiddenServiceEndpoint(object):
     def _create_listener(self, proto):
         """
         Creates the local TCP4ServerEndpoint instance, returning a
-        Deferred delivering an IPort instance that also has
+        Deferred delivering an :api:`twisted.internet.interfaces.IPort` instance that also has
         :meth:`TCP4HiddenServiceEndpoint._add_attributes` called
         against it (adds `onion_uri` and `onion_private_key` members).
         """
@@ -224,7 +223,8 @@ class TCPHiddenServiceEndpoint(object):
 
 class TorProcessProtocol(protocol.ProcessProtocol):
 
-    def __init__(self, connection_creator, progress_updates=None, config=None):
+    def __init__(self, connection_creator, progress_updates=None, config=None,
+                 ireactortime=None, timeout=None):
         """
         This will read the output from a Tor process and attempt a
         connection to its control port when it sees any 'Bootstrapped'
@@ -250,6 +250,16 @@ class TorProcessProtocol(protocol.ProcessProtocol):
         :param config: a TorConfig object to connect to the
             TorControlProtocl from the launched tor (should it succeed)
 
+        :param ireactortime:
+            An object implementing IReactorTime (i.e. a reactor) which
+            needs to be supplied if you pass a timeout.
+
+        :param timeout:
+            An int representing the timeout in seconds. If we are
+            unable to reach 100% by this time we will consider the
+            setting up of Tor to have failed. Must supply ireactortime
+            if you supply this.
+
         :ivar tor_protocol: The TorControlProtocol instance connected
             to the Tor this :api:`twisted.internet.protocol.ProcessProtocol <ProcessProtocol>`` is speaking to. Will be valid
             when the `connected_cb` callback runs.
@@ -271,6 +281,14 @@ class TorProcessProtocol(protocol.ProcessProtocol):
         self.stderr = []
         self.stdout = []
 
+        self._setup_complete = False
+        self._timeout_delayed_call = None
+        if timeout:
+            if not ireactortime:
+                raise RuntimeError('Must supply an IReactorTime object when supplying a timeout')
+            ireactortime = IReactorTime(ireactortime)
+            self._timeout_delayed_call = ireactortime.callLater(timeout, self.timeout_expired)
+
     def outReceived(self, data):
         """
         :api:`twisted.internet.protocol.ProcessProtocol <ProcessProtocol>` API
@@ -292,6 +310,14 @@ class TorProcessProtocol(protocol.ProcessProtocol):
             d.addCallback(self.tor_connected)
             d.addErrback(self.tor_connection_failed)
 
+    def timeout_expired(self):
+        """
+        A timeout was supplied during setup, and the time has run out.
+        """
+
+        self.connected_cb.errback(RuntimeError("Timed out waiting for Tor to launch."))
+        self.transport.loseConnection()
+
     def errReceived(self, data):
         """
         :api:`twisted.internet.protocol.ProcessProtocol <ProcessProtocol>` API
@@ -332,13 +358,12 @@ class TorProcessProtocol(protocol.ProcessProtocol):
 
     ## the below are all callbacks
 
-    def tor_connection_failed(self, fail):
+    def tor_connection_failed(self, failure):
         ## FIXME more robust error-handling please, like a timeout so
         ## we don't just wait forever after 100% bootstrapped (that
         ## is, we're ignoring these errors, but shouldn't do so after
         ## we'll stop trying)
         self.attempted_connect = False
-        return None
 
     def status_client(self, arg):
         args = shlex.split(arg)
@@ -352,6 +377,9 @@ class TorProcessProtocol(protocol.ProcessProtocol):
         self.progress(prog, tag, summary)
 
         if prog == 100:
+            if self._timeout_delayed_call:
+                self._timeout_delayed_call.cancel()
+                self._timeout_delayed_call = None
             self.connected_cb.callback(self)
 
     def tor_connected(self, proto):
@@ -375,9 +403,10 @@ class TorProcessProtocol(protocol.ProcessProtocol):
 
 
 def launch_tor(config, reactor,
-               tor_binary='/usr/sbin/tor',
+               tor_binary=None,
                progress_updates=None,
-               connection_creator=None):
+               connection_creator=None,
+               timeout=None):
     """
     launches a new Tor process with the given config.
 
@@ -394,7 +423,8 @@ def launch_tor(config, reactor,
     :param reactor: a Twisted IReactorCore implementation (usually
         twisted.internet.reactor)
 
-    :param tor_binary: path to the Tor binary to run.
+    :param tor_binary: path to the Tor binary to run. Tries to find the tor
+        binary if unset.
 
     :param progress_updates: a callback which gets progress updates; gets as
          args: percent, tag, summary (FIXME make an interface for this).
@@ -433,6 +463,12 @@ def launch_tor(config, reactor,
     ## config and get Tor to load that...which might be the best
     ## option anyway.
 
+    if tor_binary is None:
+        tor_binary = find_tor_binary()
+    if tor_binary is None:
+        # We fail right here instead of waiting for the reactor to start
+        raise TorNotFound('Tor binary could not be found')
+
     if config.needs_save():
         log.msg("Config was unsaved when launch_tor() called; calling save().")
         config.save()
@@ -445,6 +481,16 @@ def launch_tor(config, reactor,
         data_directory = tempfile.mkdtemp(prefix='tortmp')
         config.DataDirectory = data_directory
 
+        # Set ownership on the temp-dir to the user tor will drop privileges to
+        # when executing as root.
+        try:
+            user = config.User
+        except KeyError:
+            pass
+        else:
+            if sys.platform in ('linux2', 'darwin') and os.geteuid() == 0:
+                os.chown(data_directory, pwd.getpwnam(user).pw_uid, -1)
+
     try:
         control_port = config.ControlPort
     except KeyError:
@@ -464,7 +510,7 @@ def launch_tor(config, reactor,
     if connection_creator is None:
         connection_creator = functools.partial(TCP4ClientEndpoint(reactor, 'localhost', control_port).connect,
                                                TorProtocolFactory())
-    process_protocol = TorProcessProtocol(connection_creator, progress_updates, config)
+    process_protocol = TorProcessProtocol(connection_creator, progress_updates, config, reactor, timeout)
 
     # we set both to_delete and the shutdown events because this
     # process might be shut down way before the reactor, but if the
@@ -841,8 +887,16 @@ class TorConfig(object):
         ``things which might get into the running Tor if save() were
         to be called''
         """
-
-        return self.config[self._find_real_name(name)]
+        if name.startswith('__') and name.endswith('__'):
+            # Special case __foobar__ attributes to go for the real dict
+            # instead of self.config and raise AttributeError instead
+            # of KeyError
+            try:
+                return self.__dict__[name]
+            except KeyError, e:
+                raise AttributeError(str(e))
+        else:
+            return self.config[self._find_real_name(name)]
 
     def get_type(self, name):
         """
@@ -885,7 +939,7 @@ class TorConfig(object):
     def bootstrap(self, *args):
         try:
             self.protocol.add_event_listener('CONF_CHANGED', self._conf_changed)
-        except (RuntimeError, e):
+        except RuntimeError:
             ## for Tor versions which don't understand CONF_CHANGED
             ## there's nothing we can really do.
             log.msg("Can't listen for CONF_CHANGED event; won't stay up-to-date with other clients.")
@@ -988,7 +1042,7 @@ class TorConfig(object):
             ## auto, 0 for false and 1 for true. could be nicer if it
             ## was called AutoBoolean or something, but...
             value = value.replace('+', '_')
-            
+
             inst = None
             # FIXME: put parser classes in dict instead?
             for cls in config_types:
